@@ -54,15 +54,10 @@ static int udev_monitor_filter_remove(struct udev_monitor *udev_monitor)
 
 static GSList *device_list = NULL;
 
-static struct connman_device *find_device(const char *interface)
+static struct connman_device *find_device(int index)
 {
 	GSList *list;
-	int index;
 
-	if (interface == NULL)
-		return NULL;
-
-	index = connman_inet_ifindex(interface);
 	if (index < 0)
 		return NULL;
 
@@ -78,15 +73,16 @@ static struct connman_device *find_device(const char *interface)
 
 static void add_device(struct udev_device *udev_device)
 {
-	struct connman_device *device;
 	struct udev_list_entry *entry;
-	const char *type, *interface = NULL;
+	struct connman_device *device;
+	enum connman_device_type devtype;
+	const char *systype;
 	int index = -1;
 
 	DBG("");
 
-	type = udev_device_get_sysattr_value(udev_device, "type");
-	if (atoi(type) != 1)
+	systype = udev_device_get_sysattr_value(udev_device, "type");
+	if (systype == NULL || atoi(systype) != 1)
 		return;
 
 	entry = udev_device_get_properties_list_entry(udev_device);
@@ -97,17 +93,35 @@ static void add_device(struct udev_device *udev_device)
 			const char *value = udev_list_entry_get_value(entry);
 			if (value != NULL)
 				index = atoi(value);
-		} else if (g_str_has_prefix(name, "INTERFACE") == TRUE)
-			interface = udev_list_entry_get_value(entry);
+		}
 
 		entry = udev_list_entry_get_next(entry);
 	}
 
-	device = find_device(interface);
-	if (device != NULL)
+	if (index < 0)
 		return;
 
-	if (index < 0 || interface == NULL)
+	devtype = __connman_inet_get_device_type(index);
+
+	switch (devtype) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		return;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+		break;
+	}
+
+	device = find_device(index);
+	if (device != NULL)
 		return;
 
 	device = connman_inet_create_device(index);
@@ -124,9 +138,9 @@ static void add_device(struct udev_device *udev_device)
 
 static void remove_device(struct udev_device *udev_device)
 {
-	struct connman_device *device;
 	struct udev_list_entry *entry;
-	const char *interface = NULL;
+	struct connman_device *device;
+	int index = -1;
 
 	DBG("");
 
@@ -134,13 +148,19 @@ static void remove_device(struct udev_device *udev_device)
 	while (entry) {
 		const char *name = udev_list_entry_get_name(entry);
 
-		if (g_str_has_prefix(name, "INTERFACE") == TRUE)
-			interface = udev_list_entry_get_value(entry);
+		if (g_str_has_prefix(name, "IFINDEX") == TRUE) {
+			const char *value = udev_list_entry_get_value(entry);
+			if (value != NULL)
+				index = atoi(value);
+		}
 
 		entry = udev_list_entry_get_next(entry);
 	}
 
-	device = find_device(interface);
+	if (index < 0)
+		return;
+
+	device = find_device(index);
 	if (device == NULL)
 		return;
 
@@ -286,11 +306,9 @@ char *__connman_udev_get_devtype(const char *ifname)
 {
 	struct udev_device *device;
 	const char *devtype;
-	char syspath[128];
 
-	snprintf(syspath, sizeof(syspath) - 1, "/sys/class/net/%s", ifname);
-
-	device = udev_device_new_from_syspath(udev_ctx, syspath);
+	device = udev_device_new_from_subsystem_sysname(udev_ctx,
+							"net", ifname);
 	if (device == NULL)
 		return NULL;
 
